package com.taais.biz.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.taais.biz.domain.Data;
import com.taais.biz.domain.bo.DataBo;
import com.taais.biz.domain.vo.DataVo;
import com.taais.biz.mapper.DataMapper;
import com.taais.biz.service.IDataService;
import com.taais.common.core.core.page.PageResult;
import com.taais.common.core.utils.MapstructUtils;
import com.taais.common.orm.core.page.PageQuery;
import com.taais.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;

import static com.taais.biz.domain.table.DataTableDef.DATA;

/**
 * 数据管理Service业务层处理
 *
 * @author km
 * 2024-04-22
 */
@Service
public class DataServiceImpl extends BaseServiceImpl<DataMapper, Data> implements IDataService {
    @Resource
    private DataMapper dataMapper;

    @Override
    public QueryWrapper query() {
        return super.query().from(DATA);
    }

    private QueryWrapper buildQueryWrapper(DataBo dataBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(DATA.NAME.like
        (dataBo.getName()));
        queryWrapper.and(DATA.DATA_TYPE.eq
        (dataBo.getDataType()));
        queryWrapper.and(DATA.FILE_TYPE.eq
        (dataBo.getFileType()));
        queryWrapper.and(DATA.OBJECT_TYPE.eq
        (dataBo.getObjectType()));
        queryWrapper.and(DATA.OBJECT_SUBTYPE.eq
        (dataBo.getObjectSubtype()));
        queryWrapper.and(DATA.BATCH_NUM.eq
        (dataBo.getBatchNum()));
        queryWrapper.and(DATA.SCENE.eq
        (dataBo.getScene()));
        queryWrapper.and(DATA.DATA_SOURCE.eq
        (dataBo.getDataSource()));
        queryWrapper.and(DATA.GATHER_TIME.eq
        (dataBo.getGatherTime()));
        queryWrapper.and(DATA.GATHER_SPOT.eq
        (dataBo.getGatherSpot()));
        queryWrapper.and(DATA.URL.eq
        (dataBo.getUrl()));
        queryWrapper.and(DATA.LABELURL.eq
        (dataBo.getLabelurl()));
        queryWrapper.and(DATA.INCREMENT.eq
        (dataBo.getIncrement()));
        queryWrapper.and(DATA.LABELED.eq
        (dataBo.getLabeled()));

        return queryWrapper;
    }

    /**
     * 查询数据管理
     *
     * @param id 数据管理主键
     * @return 数据管理
     */
    @Override
    public DataVo selectById(Long id) {
            return this.getOneAs(query().where(DATA.ID.eq(id)), DataVo.class);

    }

    /**
     * 查询数据管理列表
     *
     * @param dataBo 数据管理Bo
     * @return 数据管理集合
     */
    @Override
    public List<DataVo> selectList(DataBo dataBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(dataBo);
            return this.listAs(queryWrapper, DataVo.class);
    }

    /**
     * 分页查询数据管理列表
     *
     * @param dataBo 数据管理Bo
     * @return 分页数据管理集合
     */
    @Override
    public PageResult<DataVo> selectPage(DataBo dataBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(dataBo);
            Page<DataVo> page = this.pageAs(PageQuery.build(), queryWrapper, DataVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增数据管理
     *
     * @param dataBo 数据管理Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(DataBo dataBo) {
        Data data = MapstructUtils.convert(dataBo, Data.class);

        return this.save(data);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 修改数据管理
     *
     * @param dataBo 数据管理Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(DataBo dataBo) {
        Data data =MapstructUtils.convert(dataBo, Data. class);
        if (ObjectUtil.isNotNull(data) && ObjectUtil.isNotNull(data.getId())){
            boolean updated = this.updateById(data);
                return updated;
        }
        return false;
    }

    /**
     * 批量删除数据管理
     *
     * @param ids 需要删除的数据管理主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
//    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(DATA).where(DATA.ID.in(Arrays.asList(ids)));
        return this.remove(queryWrapper);
    }

    @Override
    public List<Data> selectByIds(List<Long> ids) {
        return this.listByIds(ids);
    }

}
